/// The definition of a standard directive.
///
/// Note that this type is an input to the parser, not an output.
pub struct Directive {
    /// The provider's name for the directive.
    name: String,
    /// Description of the directive.
    doc: String,
    /// Fixed prefix term.
    prefix: String,
    /// The type of the prefix argument, if there is one.
    prefix_arg: Option<Type>,
    /// Fixed infix term.
    /// Can only be present if the prefix argument exists.
    infix: Option<String>,
    /// The type of the infix argument.
    /// Can only be present if the infix term exists.
    infix_arg: Option<Type>,
    /// Fixed suffix term.
    /// Can only be present if the infix argument exists.
    /// Note that this implies directives which have two terms have a prefix
    /// and an infix, not a prefix and a suffix.
    suffix: Option<String>,
    /// The type of the suffix argument.
    /// Can only be present if the suffix exists.
    suffix_arg: Option<Type>,
    /// The type of the elements of the list argument, if supported.
    list_arg: Option<Type>,
    // TODO(zeph): macro expansion?
}

/// Directive parameter type.
pub enum Type {
    Named {
        name: String,
        doc: String,
        underlying: Param,
    },
    Literal(Param),
}

impl Type {
    pub fn underlying(&self) -> &Param {
        match self {
            Self::Named {
                underlying,
                name: _,
                doc: _,
            } => underlying,
            Self::Literal(underlying) => underlying,
        }
    }
}

/// Types expected by directive arguments.
pub enum Param {
    /// Arbitrary text.
    Text,
    /// Speech text, possibly containing emphasis, shout, or whisper markers.
    Speech,
    /// Character in the current scene.
    Character,
    /// Duration.
    Duration,
    /// Asset requiring particular tags.
    Asset { tags: Vec<String> },
    /// Enumeration of literal options.
    Enum(Vec<String>),
}

#[cfg(test)]
mod tests {
    use super::*;
    use rstest::rstest;

    const DIRECTIVE_JSON: &str = r#"{
		"name": "kessoku",
		"prefix": "bocchi",
		"prefix_arg": 
	}"#;
}
